from django.db import models
from django.db.models.fields.files import FileField, FieldFile, ImageFieldFile
from django.utils.six import with_metaclass
from simple_seo.tags import (
    ImageMetaTag,
    TitleTag,
    MetaTag,
    KeywordsTag
)


class BaseTagField(with_metaclass(models.SubfieldBase, models.CharField)):
    """
    Base Meta Tag behaviour
    """
    description = "A hand of cards (bridge style)"

    def get_prep_value(self, value):
        prep_value = self.to_python(value)
        if prep_value.self_closed:
            return self.to_python(value).meta_content
        else:
            return self.to_python(value).tag_value

    def db_type(self, connection):
        return 'VARCHAR(%s)' % self.max_length


class BaseURLTagField(with_metaclass(models.SubfieldBase, models.URLField)):
    """
    Base URL Meta Tag behaviour
    """
    description = "A hand of cards (bridge style)"

    def get_prep_value(self, value):
        prep_value = self.to_python(value)
        if prep_value.self_closed:
            return self.to_python(value).meta_content
        else:
            return self.to_python(value).tag_value


class BaseImageTagField(with_metaclass(models.SubfieldBase, models.ImageField)):
    """
    Base Image Meta Tag behaviour
    """
    description = "A hand of cards (bridge style)"

    def get_prep_value(self, value):
        prep_value = self.to_python(value)
        if prep_value.self_closed:
            return self.to_python(value).meta_content
        else:
            return self.to_python(value).tag_value

    def pre_save(self, model_instance, add):
        "Returns field's value just before saving."
        file = FieldFile(model_instance, self, super(FileField, self).pre_save(model_instance, add).meta_content)
        tag = super(FileField, self).pre_save(model_instance, add)  # Let's hack this
        if hasattr(tag, '_inmemoryuploadedfile') and getattr(tag, '_inmemoryuploadedfile'):
            file.save(file.name, tag._inmemoryuploadedfile, save=False)
        return file


class TitleTagField(with_metaclass(models.SubfieldBase, BaseTagField)):
    """
    Creates a field for Title Tag
    * Not Null
    * Not Blank
    * Max-length 68
    """
    description = "Field for Storing <title> tag"

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 68
        kwargs['blank'] = False
        kwargs['db_index'] = False
        kwargs['null'] = False
        super(TitleTagField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if isinstance(value, TitleTag):
            return value
        title = super(TitleTagField, self).to_python(value)
        title_tag = TitleTag(**{'value': title})
        return title_tag


class MetaTagField(with_metaclass(models.SubfieldBase, BaseTagField)):
    """
    Creates a field for Meta Tag
    * Max-length 255
    """
    description = "Field for Storing <meta /> tag"

    def __init__(self, *args, **kwargs):
        kwargs['blank'] = True
        kwargs['db_index'] = False
        kwargs['null'] = True
        kwargs['max_length'] = 255
        super(MetaTagField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if isinstance(value, MetaTag):
            return value
        content = super(MetaTagField, self).to_python(value)
        meta_tag = MetaTag(**{'name': self.name, 'value': content})
        return meta_tag


class URLMetaTagField(with_metaclass(models.SubfieldBase, BaseURLTagField)):
    """
    Creates a field for URL Meta Tag
    """
    description = "Field for Storing <meta /> tag"

    def __init__(self, *args, **kwargs):
        kwargs['blank'] = True
        kwargs['db_index'] = False
        kwargs['null'] = True
        super(URLMetaTagField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if isinstance(value, MetaTag):
            return value
        content = super(URLMetaTagField, self).to_python(value)
        meta_tag = MetaTag(**{'name': self.name, 'value': content})
        return meta_tag


class ImageMetaTagField(with_metaclass(models.SubfieldBase, BaseImageTagField)):
    """
    Creates a field for Image Meta Tag
    """
    description = "Field for Storing <meta /> tag"

    def __init__(self, *args, **kwargs):
        kwargs['blank'] = True
        kwargs['db_index'] = False
        kwargs['null'] = True
        super(ImageMetaTagField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if isinstance(value, ImageMetaTag):
            return value
        content = super(ImageMetaTagField, self).to_python(value)
        image_meta_tag = ImageMetaTag(**{'name': self.name, 'value': content, 'path': self.upload_to})
        return image_meta_tag


class KeywordsTagField(with_metaclass(models.SubfieldBase, BaseTagField)):
    """
    Creates a field for Keywords Meta Tag
    * Max-length 255
    """
    description = "Field for Storing <meta name='keywords' /> tag"

    def __init__(self, *args, **kwargs):
        kwargs['blank'] = True
        kwargs['db_index'] = False
        kwargs['null'] = True
        kwargs['max_length'] = 255
        self.name = 'keywords'
        super(KeywordsTagField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if isinstance(value, KeywordsTag):
            return value
        keywords = super(KeywordsTagField, self).to_python(value)
        keyword_tag = KeywordsTag(**{'name': self.name, 'value': keywords})
        return keyword_tag


from django import template
from django.core.urlresolvers import resolve
from django.core.cache import cache
from django.utils import translation
import logging

from .. import SEO_CACHE_PREFIX, SEO_CACHE_TIMEOUT, SEO_USE_CACHE, get_model_for_view
from ..fields import (
    TitleTagField,
    MetaTagField,
    KeywordsTagField
)


log = logging.getLogger(__name__)

register = template.Library()


def _build_prefix(context, view_name):
    lang = translation.get_language()
    return SEO_CACHE_PREFIX + ':' + view_name + ':' + lang + ':' + context['request'].path


class MetadataNode(template.Node):
    """
    Template Tag node for Metadata
    * gets the view name from request
    * obtains metadata model and object
    * print each supported field
    """

    def render(self, context):
        view_name = resolve(context['request'].path).url_name  # resolve view name

        # Check if metadata is in cache
        if SEO_USE_CACHE:
            metadata_html = cache.get(_build_prefix(context, view_name))
            if metadata_html:
                return metadata_html

        seo_model = get_model_for_view(view_name)
        try:
            metadata = seo_model.objects.get(view_name=view_name)
            metadata_html = ""
            for field in metadata._meta.fields:
                if isinstance(field, (TitleTagField, MetaTagField, KeywordsTagField)):
                    metadata_html += field.to_python(getattr(metadata, field.name)).print_tag() + "\n"
                else:
                    pass
            if metadata_html != "" and SEO_USE_CACHE:
                cache.set(_build_prefix(context, view_name), metadata_html, SEO_CACHE_TIMEOUT)

            return metadata_html
        except seo_model.DoesNotExist as exc:
            # Skipping error to avoid breaking the view
            log.exception("No metadata found for view %s" % view_name)


@register.tag
def view_metadata(context, parser):
    return MetadataNode()




"""
Load settings
"""
from django.conf import settings
from django.utils.six import iteritems

from simple_seo.models import BaseMetadata


SEO_CACHE_PREFIX = getattr(settings, 'SEO_CACHE_PREFIX', 'simple_seo:')
SEO_CACHE_TIMEOUT = getattr(settings, 'SEO_CACHE_PREFIX', 60 * 60 * 24)
SEO_USE_CACHE = getattr(settings, 'SEO_CACHE_PREFIX', False)

_simple_seo_registry = []


def register(model, views=None):
    """
    Registers model for the given views. If no views given then all views are bound.
    If more than one seo model is registered, order matter (in case view name is defined in more than one model,
    first registered is applied only
    :param model: Model to register
    :param views: list of str
    :return:
    """
    if not issubclass(model, BaseMetadata):
        raise TypeError("Model must be an instance of simple_seo.models.BaseMetadata")

    global _simple_seo_registry
    if views and not isinstance(views, (str, list, tuple)):
        raise TypeError("Views must be of type of (str, list, tuple). I.e. 'myview', ('view1', 'view2), ['v1', 'v2']")
    if not views or not len(views):
        views = 'ALL'
    _simple_seo_registry.append(
        {
            model: views
        }
    )


def get_model_for_view(view):
    """
    Given a view name, give a valid seo model. It's gonna be the first model who contains the view name or has
     all-views scope
    :param view:
    :return: model
    """
    if not view:
        raise ValueError("View must have value")

    global _simple_seo_registry
    for registry_dict in _simple_seo_registry:
        for key, value in iteritems(registry_dict):
            if isinstance(value, (list, tuple)):
                if view in value:
                    return key
            elif isinstance(value, str) and value == 'ALL':
                return key
    raise ValueError("No model has been found for view with name %s. Have you registered one?" % view)


from __future__ import unicode_literals
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.utils.encoding import python_2_unicode_compatible
from django.contrib.staticfiles.storage import staticfiles_storage


@python_2_unicode_compatible
class BaseTag(object):
    """
    Base Tag behaviour
    """
    tag_name = ""
    self_closed = True  # Defines if should close itself < />
    meta_name = ""
    meta_content = ""
    tag_value = ""

    def print_tag(self):
        """
        Builds tag as text for printing
        :return: text
        """
        if not isinstance(self, BaseTag):
            raise TypeError("Tag must be of class simple-seo.tags.BaseTag")

        # print_attrs = ""
        # for attr_name, attr_value in iteritems(self._attributes):
        #     print_attrs += " %s=\"%s\"" % (attr_name, attr_value)

        if self.self_closed:
            if self.meta_name and self.meta_content:
                return "<%s name='%s' content='%s' />" % (self.tag_name, self.meta_name, self.meta_content)
            else:
                return "<%s/>" % self.tag_name
        else:
            if self.meta_name and self.meta_content:
                return "<%s name='%s' content='%s'>%s</%s>" % (self.tag_name, self.meta_name, self.meta_content, self.tag_value, self.tag_name)
            else:
                return "<%s>%s</%s>" % (self.tag_name, self.tag_value, self.tag_name)

    def __str__(self):
        if self.self_closed:
            return self.meta_content
        else:
            return self.tag_value

    def __len__(self):
        raise NotImplementedError("Must implement tag output __len__()")


class TitleTag(BaseTag):
    """
    Title Tag class
    """

    def __init__(self, *args, **kwargs):
        self.tag_name = "title"
        self.self_closed = False
        if 'value' in kwargs:
            if len(kwargs['value']) > 68:
                self.tag_value = kwargs['value'][:68]
            else:
                self.tag_value = kwargs['value']

    def __str__(self):
        return self.tag_value

    def __len__(self):
        return len(self.tag_value)


class BaseMetatag(BaseTag):
    """
    Base Meta Tag
    """

    def __init__(self, *args, **kwargs):
        self.tag_name = 'meta'
        self.self_closed = True
        if 'name' in kwargs:
            self.meta_name = kwargs['name']

    def __unicode__(self):
        return self.meta_content

    def __str__(self):
        return self.meta_content

    def __len__(self):
        return len(self.meta_content)


class MetaTag(BaseMetatag):
    """
    Meta Tag class
    """

    def __init__(self, *args, **kwargs):
        super(MetaTag, self).__init__(*args, **kwargs)
        if 'value' in kwargs:
            if kwargs['value'] and len(kwargs['value']) > 255:
                self.meta_content = kwargs['value'][:255]
            else:
                self.meta_content = kwargs['value']


class ImageMetaTag(BaseMetatag):
    """
    Image Meta Tag class
    """

    def __init__(self, *args, **kwargs):
        super(ImageMetaTag, self).__init__(*args, **kwargs)
        if 'value' in kwargs:
            if kwargs['value']:
                if isinstance(kwargs['value'], InMemoryUploadedFile):
                    rel_path = kwargs['path'] + kwargs['value'].name
                    self._inmemoryuploadedfile = kwargs['value']
                else:
                    self._inmemoryuploadedfile = None
                    rel_path = kwargs['value']
                if staticfiles_storage:
                    self.meta_content = staticfiles_storage.url(rel_path)
                else:
                    self.meta_content = rel_path
    @property
    def url(self):
        return self.meta_content

    def __str__(self):
        return str(self.meta_content)


class KeywordsTag(BaseMetatag):
    """
    Keywords Meta Tag class
    """

    def _clean(self, value):
        if value:
            return value.replace('"', '&#34;').replace("\n", ", ").strip()
        else:
            return value

    def __init__(self, *args, **kwargs):
        super(KeywordsTag, self).__init__(*args, **kwargs)
        if 'value' in kwargs:
            if kwargs['value'] and len(kwargs['value']) > 255:
                self.meta_content = self._clean(kwargs['value'][:255])
            else:
                self.meta_content = self._clean(kwargs['value'])



import os

from setuptools import setup


README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-simple-seo',
    version='0.2.3',
    packages=['simple_seo', 'simple_seo.templatetags'],
    include_package_data=True,
    license='MIT License',  # example license
    description='Simple SEO Module for Django',
    long_description=README,
    url='http://codeispoetry.me/',
    author='danigosa',
    author_email='danigosa@gmail.com',
    install_requires=[

    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Operating System :: OS Independent",
        "Environment :: Web Environment",
        "Framework :: Django",
        "License :: OSI Approved :: BSD License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
)
